package xyz.veronie.bgg.result;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map.Entry;

import org.eclipse.core.runtime.preferences.InstanceScope;
import org.osgi.service.prefs.BackingStoreException;
import org.osgi.service.prefs.Preferences;

import xyz.veronie.bgg.types.FamilyType;
import xyz.veronie.bgg.types.FilterFlagState;
import xyz.veronie.bgg.types.SourceFilter;
import xyz.veronie.bgg.types.Subtype;
import xyz.veronie.bgg.types.UserFlag;

/// DTO / container for configuration of result download. Also defines startup settings for controls.
/// This container is handled by the ResultConfigManager.
public class ResultConfig {
	

	// change version if you change any of the constants or enum strings
	private static final String VERSION = "1.0";
	private static final String XYZ_VERONI_BGG = "xyz.veroni.bgg";

	private static final String USER_NAME = "UserName";
	private static final String USER_FLAGS = "UserFlags";
	private static final String RESULT_CONFIG = "ResultConfig";

	private static final String FAMILY_TYPE_KEY = "FamilyType";
	private static final String SUBTYPE_KEY = "Subtype";
	private static final String VERSION_KEY = "Version";
	private static final String SOURCE_FILTER_KEY = "SourceFilter";
	
	public SourceFilter source = SourceFilter.BGG_USER;
	public Subtype subtype = Subtype.BOARDGAME;

	// bgg user filter settings
	public String user = "";
	public HashMap<UserFlag, FilterFlagState> userFlags = new HashMap<UserFlag, FilterFlagState>() {
		private static final long serialVersionUID = 3086538711393696853L;

	{
		for (UserFlag uf : UserFlag.values()) {
			put(uf, FilterFlagState.IGNORE);
		}
		put(UserFlag.OWN, FilterFlagState.IS);
		put(UserFlag.PREVIOUSLY_OWNED, FilterFlagState.ISNOT);
	}};
	
	public Integer geeklistId = null;
	public Integer familyId = null;
	public FamilyType familyType = FamilyType.ALL;
	
	public ResultConfig() {
		initFromPrefrences();
	}
	
	@Override
	public String toString() {
		switch(source) {
		case BGG_USER:
			return "user = " + user;
		case GEEKLIST:
			return "Geeklist #" + geeklistId;
		case FAMILY:
			return "Family #" + familyId;
		default:
			return "";
		}
	}
	
	
	public void saveToPreferences() {
		// We access the instanceScope
		Preferences preferences = InstanceScope.INSTANCE.getNode(XYZ_VERONI_BGG);

		Preferences prefResultConfig = preferences.node(RESULT_CONFIG);
		prefResultConfig.put(VERSION_KEY, VERSION);
		prefResultConfig.put(SOURCE_FILTER_KEY, source.name());
		prefResultConfig.put(SUBTYPE_KEY, subtype.name());
		prefResultConfig.put(USER_NAME, user);
		prefResultConfig.put(SourceFilter.GEEKLIST.name(), (geeklistId == null) ? "" : geeklistId.toString());
		prefResultConfig.put(SourceFilter.FAMILY.name(), (familyId == null) ? "" : familyId.toString());
		prefResultConfig.put(FAMILY_TYPE_KEY, familyType.name());
		
		Preferences prefUserFlags = preferences.node(USER_FLAGS);
		Iterator<Entry<UserFlag, FilterFlagState>> it = userFlags.entrySet().iterator();
	    while (it.hasNext()) {
	        Entry<UserFlag, FilterFlagState> pair = it.next();
	        prefUserFlags.put(pair.getKey().toString(), pair.getValue().name());
	    }

		try {
		    // forces the application to save the preferences
		    preferences.flush();
	    } catch (BackingStoreException e) {
	        e.printStackTrace();
	    }
	}

	public void initFromPrefrences() {
		Preferences preferences = InstanceScope.INSTANCE.getNode(XYZ_VERONI_BGG);
		if(preferences != null) {
			Preferences prefResultConfig = preferences.node(RESULT_CONFIG);

			// assume latest version
			String version = prefResultConfig.get(VERSION_KEY, VERSION);
			if(!version.equals(VERSION)) {
				System.out.println("WARN: Version mismatch. Trying to read config anyways.");
				return;
			}
			
			source = SourceFilter.getEnum(prefResultConfig.get(SOURCE_FILTER_KEY, source.name()));
			subtype = Subtype.getEnum(prefResultConfig.get(SUBTYPE_KEY, subtype.name()));
			user = prefResultConfig.get(USER_NAME, user);
			String geeklist = prefResultConfig.get(SourceFilter.GEEKLIST.name(), "");
			if(!geeklist.isEmpty()) {
				geeklistId = Integer.valueOf(geeklist);
			} 
			String family = prefResultConfig.get(SourceFilter.FAMILY.name(), "");
			if(!family.isEmpty()) {
				familyId = Integer.valueOf(family);
			}
			familyType = FamilyType.getEnum(prefResultConfig.get(FAMILY_TYPE_KEY, familyType.name()));
			
			Preferences prefUserFlags = preferences.node(USER_FLAGS);
			Iterator<Entry<UserFlag, FilterFlagState>> it = userFlags.entrySet().iterator();
		    while (it.hasNext()) {
		        Entry<UserFlag, FilterFlagState> pair = it.next();
		        String storedUserFlagValue = prefUserFlags.get(pair.getKey().toString(), pair.getValue().name());
		        pair.setValue(FilterFlagState.getEnum(storedUserFlagValue));
		    }
		}

	}
	
	
}
