package xyz.veronie.bgg.result;


import javax.inject.Inject;
import javax.inject.Singleton;

import org.eclipse.e4.core.di.annotations.Creatable;
import org.eclipse.e4.core.di.annotations.Optional;
import org.eclipse.e4.ui.di.UIEventTopic;

import xyz.veronie.bgg.types.EventConstants;
import xyz.veronie.bgg.types.FamilyType;
import xyz.veronie.bgg.types.SourceFilter;
import xyz.veronie.bgg.types.Subtype;
import xyz.veronie.bgg.types.UserFlagEvent;


/// This object/instance is responsible for managing the current state of the configuration for 
/// BGG result downloads. It is sent an event whenever one of the controls for the 
/// configuration changes, and modifies the ResultConfig accordingly.
/// Other parts of the program can ask for the result config to either display its state
/// or execute the BGG download. 
@Creatable
@Singleton
public class ResultConfigManager {

	private ResultConfig resultConfig;

	public ResultConfigManager() {
		resultConfig = new ResultConfig();
	}
	
	// TODO: store only the config value that changed instead of the whole config...
	
	@Inject
	@Optional
	private void subscribeTopicSourceChanged
	    (@UIEventTopic(EventConstants.TOPIC_SOURCE_CHANGED)
	        SourceFilter source) {
	    getResultConfig().source = source;
	    storePreferences();
	    System.out.println("TOPIC_SOURCE_CHANGED: source = " + source);
	}
	
	@Inject
	@Optional
	private void subscribeTopicSubtypesChanged
	    (@UIEventTopic(EventConstants.TOPIC_SUBTYPE_CHANGED)
	    Subtype subtype) {
	    getResultConfig().subtype = subtype;
	    storePreferences();
	    System.out.println("TOPIC_SUBTYPE_CHANGED: " + subtype);
	}
	
	@Inject
	@Optional
	private void subscribeTopicUserChanged
	    (@UIEventTopic(EventConstants.TOPIC_USER_CHANGED)
	        String user) {
		System.out.println("set user to '" + user + "'");
	    getResultConfig().user = user;
	    storePreferences();
	    System.out.println("TOPIC_USER_CHANGED: user = " + user);
	}
	
	@Inject
	@Optional
	private void subscribeTopicUserFlagChanged
	    (@UIEventTopic(EventConstants.TOPIC_USERFLAG_CHANGED)
	        UserFlagEvent e) {
	    getResultConfig().userFlags.put(e.flag, e.state);
	    storePreferences();
	    System.out.println("TOPIC_USERFLAG_CHANGED: " + e.flag + " " + e.state);
	}
	
	@Inject
	@Optional
	private void subscribeTopicResultGeeklistChanged
	    (@UIEventTopic(EventConstants.TOPIC_GEEKLIST_CHANGED)
	    	Integer id) {
	    getResultConfig().geeklistId = id;
	    storePreferences();
	    System.out.println("TOPIC_GEEKLIST_CHANGED: geeklistId = " + id);
	}

	@Inject
	@Optional
	private void subscribeTopicResultFamilyChanged
	    (@UIEventTopic(EventConstants.TOPIC_FAMILY_CHANGED)
	    	Integer id) {
	    getResultConfig().familyId = id;
	    storePreferences();
	    System.out.println("TOPIC_FAMILY_CHANGED: geeklistId = " + id);
	}
	
	@Inject
	@Optional
	private void subscribeTopicResultFamilyChanged
	    (@UIEventTopic(EventConstants.TOPIC_FAMILYTYPE_CHANGED)
	    	FamilyType ft) {
	    getResultConfig().familyType = ft;
	    storePreferences();
	    System.out.println("TOPIC_FAMILYTYPE_CHANGED: family type = " + ft);
	}
	
	public ResultConfig getResultConfig() {
		return resultConfig;
	}

	
	private void storePreferences() {
		resultConfig.saveToPreferences();
	}
	
}
