package xyz.veronie.bgg.result;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.nio.file.Files;
import java.nio.file.LinkOption;
import java.nio.file.Path;
import java.nio.file.Paths;

import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.graphics.ImageLoader;

import xyz.veronie.bgg.result.internal.OutputResultHelper;
import xyz.veronie.bgg.ui.helpers.Resources;

public class Thing {

	private int id;
	private ThingMetaData metaData;
	private ThingUserData userData;
	private ThingDetails details;
	
	// cache
	private Image thumbImage;
	private Path imagePath;
		
	public static final String IdHeader = "ID";
	public static final String ThumbHeader = "Thumbnail";
	public static final String NameHeader = "Name";
	public static final String DetailsHeader = "Details";
	public static final String ExportFlagHeader = "Export";
	
	public Thing(int id, ThingMetaData metaData) {
		this.id = id;
		this.metaData = metaData;
	}
	
	
	public int getId() {
		return id;
	}
	public void setId(int id) {
		this.id = id;
	}
	public ThingMetaData getMetaData() {
		return metaData;
	}
	public void setMetaData(ThingMetaData metaData) {
		this.metaData = metaData;
	}
	
	public void setUserData(ThingUserData userData) {
		this.userData = userData;
	}
	
	public ThingUserData getUserData() {
		return userData;
	}
	
	public ThingDetails getDetails() {
		return details;
	}
	
	public void setDetails(ThingDetails details) {
		this.details = details;
	}
	
	public Image getThumbnail() {
		if(thumbImage == null) {
			getThumbImage(id);
		}
		return thumbImage;
	}
	
	public String getImagePath() {
		if(imagePath != null)
			return imagePath.toString();
		else 
			return Paths.get(Resources.INSTANCE.getThumbsDir().toString()
					+ File.separator + String.valueOf(id) + ".png").toString();
	}
	
	private void getThumbImage(int thingId) {
		Path tmpDir = Resources.INSTANCE.getTmpDir();
		if(tmpDir != null) {
			imagePath = Paths.get(Resources.INSTANCE.getThumbsDir().toString()
					+ File.separator + String.valueOf(thingId) + ".png");
			
			if(Files.exists(imagePath, LinkOption.NOFOLLOW_LINKS)) {
				String filename = imagePath.toString().replace("\\","/");
				thumbImage = Resources.INSTANCE.getResourceManager().createImage(
						ImageDescriptor.createFromFile(null, filename));				
			} else {
				try {
					ImageDescriptor descriptor = ImageDescriptor.createFromURL(
							new URL(metaData.getThumbURL()));
					thumbImage = descriptor.createImage();

					ImageLoader saver = new ImageLoader();
					saver.data = new ImageData[] { thumbImage.getImageData() };
					saver.save(imagePath.toString(), SWT.IMAGE_PNG);					
				} catch (MalformedURLException e) {
					System.out.println("INFO: Malformed URL for ThingId " + thingId);
				}
			}

		}
	}


	/// get field at idx, order of titles
    public String getField(int idx) {
    	
    	String returnStr;

    	switch(idx) {
    	case 0:
    		returnStr = "";
    		break;
    	case 1:
    		returnStr = this.getMetaData().getName();
    		break;
    	case 2:
    		returnStr = String.valueOf(this.id);
    		break;
    	case 3:
    		returnStr = (this.getDetails() != null) ? "yes" : "no";
    		break;
    	case 4:
    		returnStr = this.getMetaData().getName();
    		break;
    	default:
    		throw new ArrayIndexOutOfBoundsException(
    				"idx " + String.valueOf(idx) + " must be in [0,4]");
    	}

    	return returnStr;
    }
    
    
    @Override
    public boolean equals(Object o) {
    	if(o instanceof Thing) {
    		return ((Thing)o).getId() == getId();
    	} else {
    		return false;
    	}
    }
	
    
    private static String resultHeader() {
    	String header = "id,name,image,";
    	return header;
    }
    
    public static String makeResultHeader() {
    	String header = Thing.resultHeader() + ThingUserData.resultHeader() + ThingDetails.resultHeader();
    	return header;
    }
    
    /// convert the thing into a comma-separated line for a result.txt file
    /**
     * The format is (without line breaks):
     * id,name,designer,
     * 
     * owned,trading,wanting,wishing,userrating,comment,numplays,wishpriority
     * 
     * designer,publisher,artist,yearpublished,minplayers,maxplayers,playingtime,minplaytime,maxplaytime,
     * age,usersrated,average,bayesaverage,rank,rank_wg,numcomments,numweights,averageweight,stddev,median,
     * category,mechanic,
     * 1player,2player,3player,4player,5player,6player,7player,8player,9player,10player,
     * 11player,12player,13player,14player,15player,16player,17player,18player,19player,20player,
     * description,exp,basegame,reimplement,reimplement_name,reimplemented,reimplemented_name,
     * contains,contains_name,iscontained,iscontained_name,integration,integration_name,price,
     * expansions,domain,family,age_poll
     * 
     * @return
     */
    // TODO: rewrite: order isn't important, column names are
    public String toResultTxtLine() {
    	StringBuilder str = new StringBuilder();

        // id,name,image,
    	OutputResultHelper.appendToResult(str, id);
    	OutputResultHelper.appendToResult(str, metaData.getName());
    	OutputResultHelper.appendToResult(str, getImagePath());
    	
    	if(userData != null) {
    		userData.appendToResult(str);
    	} else {
    		str.append(ThingUserData.emptyReplacerLine());
    	}
    	
    	if(details != null) {
    		str.append(details.toResultTextLine());
    	} else {
    		str.append(ThingDetails.emptyReplacerLine());
    	}
    	
    	return str.toString();
    }




}
