package xyz.veronie.bgg.ui.dialogs;

import java.util.List;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.wb.swt.SWTResourceManager;

import xyz.veronie.bgg.ui.helpers.BatColors;

public class LoadGameListDialog extends Dialog {
	private String selectedName;

	private List<String> thingLists;

	/**
	 * Create the dialog.
	 * @param parentShell
	 * @param handlerService 
	 * @param commandService 
	 */
	public LoadGameListDialog(Shell parentShell, List<String> thingLists) 
	{
		super(parentShell);
		this.thingLists = thingLists;
		setShellStyle(SWT.BORDER | SWT.RESIZE | SWT.APPLICATION_MODAL);
	}

	/**
	 * Create contents of the dialog.
	 * 
	 * @param parent
	 */
	@Override
	protected Control createDialogArea(Composite parent) {
		parent.setBackground(BatColors.getButtonBgColor());
		Composite container = (Composite) super.createDialogArea(parent);
		container.setFont(SWTResourceManager.getFont("Segoe UI", 11, SWT.NORMAL));
		container.setBackground(BatColors.getButtonBgColor());
		GridLayout gl_container = new GridLayout(1, false);
		gl_container.verticalSpacing = 24;
		gl_container.marginWidth = 24;
		gl_container.marginTop = 24;
		gl_container.marginRight = 24;
		gl_container.marginLeft = 24;
		gl_container.marginHeight = 24;
		gl_container.marginBottom = 24;
		gl_container.horizontalSpacing = 24;
		container.setLayout(gl_container);
		
		Label lblEnterAName = new Label(container, SWT.NONE);
		lblEnterAName.setForeground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		lblEnterAName.setFont(SWTResourceManager.getFont("Segoe UI", 11, SWT.NORMAL));
		lblEnterAName.setText("Select a game list to replace the current one:");
		lblEnterAName.setBackground(BatColors.getButtonBgColor());
		
		
		TableViewer tableViewer = new TableViewer(container, SWT.BORDER | SWT.FULL_SELECTION );
		Table table = tableViewer.getTable();
		table.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseDoubleClick(MouseEvent e) {
				closeDialogOk();
			}
		});
		table.setLinesVisible(true);
//		gd_tableGameList.heightHint = 466;
		table.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		table.setFont(SWTResourceManager.getFont("Segoe UI", 11, SWT.NORMAL));
		
		final TableViewerColumn viewerColumn = new TableViewerColumn(tableViewer, SWT.NONE);
        final TableColumn column = viewerColumn.getColumn();
        column.setText("Game list name");
        //column.setWidth(150);
        column.setAlignment(SWT.LEFT);
        column.setResizable(true);
        column.setMoveable(true);
        
        viewerColumn.setLabelProvider(new ColumnLabelProvider() {
        	@Override
            public String getText(Object element) {
                return (String) element;
            }
        });
    	
		tableViewer.setContentProvider(new ArrayContentProvider());
		tableViewer.setInput(thingLists);
		int selection = 0;
		tableViewer.setSelection(new StructuredSelection(tableViewer.getElementAt(selection)), true);
		selectedName = (String)tableViewer.getStructuredSelection().getFirstElement();
		
		tableViewer.addSelectionChangedListener(new ISelectionChangedListener() {

			@Override
			public void selectionChanged(SelectionChangedEvent event) {
				selectedName = (String)event.getStructuredSelection().getFirstElement();
			}
			
		});

		column.pack();
		
		Composite otherActionsComposite = new Composite(container, SWT.NONE);
		otherActionsComposite.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
		otherActionsComposite.setLayout(new GridLayout(1, false));
	
		return container;
	}


	private void closeDialogOk() {
		this.setReturnCode(OK);
		this.close();
	}
	

	/**
	 * Create contents of the button bar.
	 * @param parent
	 */
	@Override
	protected void createButtonsForButtonBar(Composite parent) {
		parent.setBackground(BatColors.getButtonBgColor());
		
		createButton(parent, IDialogConstants.OK_ID, IDialogConstants.OK_LABEL, true);
		createButton(parent, IDialogConstants.CANCEL_ID, IDialogConstants.CANCEL_LABEL, false);
	}

	/**
	 * Return the initial size of the dialog.
	 */
	@Override
	protected Point getInitialSize() {
		return new Point(500, 500);
	}

	public String getSelectedName() {
		return selectedName;
	}
	
}
