package xyz.veronie.bgg.ui.filters;

import java.util.ArrayList;
import java.util.List;

import javax.annotation.PostConstruct;
import javax.inject.Inject;

import org.eclipse.e4.core.di.annotations.Creatable;
import org.eclipse.e4.core.di.annotations.Optional;
import org.eclipse.e4.core.services.events.IEventBroker;
import org.eclipse.e4.ui.di.UIEventTopic;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;

import xyz.veronie.bgg.result.ResultConfigManager;
import xyz.veronie.bgg.types.EventConstants;
import xyz.veronie.bgg.types.FamilyType;

/// These are the controls to retrieve thing IDs for a given family ID
@Creatable
public class FamilySourceFilter {

	@Inject private IEventBroker eventBroker;
	@Inject private ResultConfigManager configManager;
	private StyledText confLabel;
	private Combo cbFamilyId;
	
	@PostConstruct
	public void postConstruct(Composite parent) {
		Composite mainCompo = new Composite(parent, SWT.FILL);
		GridLayout gl_mainCompo = new GridLayout(2, false);
		mainCompo.setLayout(gl_mainCompo);
		mainCompo.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		        
        Label lblFamily = new Label(mainCompo, SWT.LEFT);
        lblFamily.setText("Family ID: ");
        lblFamily.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false));
		        
        cbFamilyId = new Combo(mainCompo, SWT.DROP_DOWN);
        cbFamilyId.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false));
        
        Integer fid = configManager.getResultConfig().familyId;
        if(fid != null) {
        	cbFamilyId.setText(Integer.toString(fid));
        }

        cbFamilyId.addSelectionListener(new SelectionAdapter() {
            public void widgetDefaultSelected(SelectionEvent e) {
            	try {
	            	setFamilyId(Integer.parseInt(cbFamilyId.getText()));
	            	System.out.println("set family id to '" + configManager.getResultConfig().familyId + "'");					
				} 
				catch (NumberFormatException ex) {
					eventBroker.send(EventConstants.TOPIC_STATUS, "[ERROR] GeeklistId must be a whole number.");
				}
            	System.out.println("set family id to '" + configManager.getResultConfig().familyId + "'");
            }
        });
        cbFamilyId.addFocusListener(new FocusListener() {
			
			@Override
			public void focusLost(FocusEvent e) {
				try {
	            	setFamilyId(Integer.parseInt(cbFamilyId.getText()));
	            	System.out.println("set family id to '" + configManager.getResultConfig().geeklistId + "'");					
				} 
				catch (NumberFormatException ex) {
					eventBroker.send(EventConstants.TOPIC_STATUS, "[ERROR] GeeklistId must be a whole number.");
				}
            	System.out.println("set family id to '" + configManager.getResultConfig().familyId + "'");
			}
			
			@Override
			public void focusGained(FocusEvent e) {
				// nothing
			}
		});
        
        
        Label lblFamilyType = new Label(mainCompo, SWT.LEFT);
        lblFamilyType.setText("Family Type: ");
        lblFamilyType.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false));
		 
        ComboViewer cbSubtypes = new ComboViewer(mainCompo, SWT.READ_ONLY);
		cbSubtypes.getCombo().setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false));
		cbSubtypes.setContentProvider(ArrayContentProvider.getInstance());
		List<FamilyType> subtypes = new ArrayList<FamilyType>();
		for (FamilyType ft : FamilyType.values()) { 
			subtypes.add(ft);
		}
		cbSubtypes.setInput(subtypes);
		cbSubtypes.setSelection(new StructuredSelection(configManager.getResultConfig().familyType));
		cbSubtypes.addSelectionChangedListener(new ISelectionChangedListener() {
			@Override
			public void selectionChanged(SelectionChangedEvent event) {
				IStructuredSelection selection = (IStructuredSelection) event.getSelection();
				eventBroker.send(EventConstants.TOPIC_FAMILYTYPE_CHANGED, selection.getFirstElement());
			}
		});
		
		Label lblInfo = new Label(mainCompo, SWT.LEFT);
		lblInfo.setText("Description: ");
		lblInfo.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false));
		
        confLabel = new StyledText(mainCompo, SWT.MULTI | SWT.READ_ONLY | SWT.BORDER);
        confLabel.setText("");
        confLabel.setWordWrap(true);
        confLabel.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));      
        
        mainCompo.pack();
        mainCompo.layout(true, true);
	}


	public void setFamilyId(int familyId) {
		if(eventBroker != null) {
			eventBroker.post(EventConstants.TOPIC_FAMILY_CHANGED, familyId);
		} else {
		   	System.out.println("setFamilyId: eventBroker is null.");
		}
	}
	
	@Inject
	@Optional
	private void subscribeTopicGeeklistInfo
	    (@UIEventTopic(EventConstants.TOPIC_FAMILY_INFO)
	        String familyInfo) {
		System.out.println("TOPIC_FAMILY_INFO: " + familyInfo);
		confLabel.setText(familyInfo);
		confLabel.redraw();
	}
}
