package xyz.veronie.bgg.ui.filters;

import javax.annotation.PostConstruct;
import javax.inject.Inject;

import org.eclipse.e4.core.di.annotations.Creatable;
import org.eclipse.e4.core.di.annotations.Optional;
import org.eclipse.e4.core.services.events.IEventBroker;
import org.eclipse.e4.ui.di.UIEventTopic;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;

import xyz.veronie.bgg.result.ResultConfigManager;
import xyz.veronie.bgg.types.EventConstants;

/// These are the controls to retrieve thing IDs for a given family ID
@Creatable
public class GeeklistSourceFilter {
	
	@Inject private IEventBroker eventBroker;
	@Inject private ResultConfigManager configManager;
	
	private StyledText confLabel;
	private Combo cbGeeklistId;
	
	@PostConstruct
	public void postConstruct(Composite parent) {
		Composite mainCompo = new Composite(parent, SWT.FILL);
		GridLayout gl_mainCompo = new GridLayout(2, false);
		mainCompo.setLayout(gl_mainCompo);
		mainCompo.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
        
        Label lblGeeklist = new Label(mainCompo, SWT.LEFT);
        lblGeeklist.setText("Geeklist ID: ");
        lblGeeklist.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false));
		        
        cbGeeklistId = new Combo(mainCompo, SWT.DROP_DOWN);
        Integer gid = configManager.getResultConfig().geeklistId;
        if(gid != null) {
        	cbGeeklistId.setText(Integer.toString(gid));
        }
        cbGeeklistId.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false));
        cbGeeklistId.addSelectionListener(new SelectionAdapter() {
            public void widgetDefaultSelected(SelectionEvent e) {
            	try {
					setGeeklistId(Integer.parseInt(cbGeeklistId.getText()));
	            	System.out.println("set geeklist id to '" + configManager.getResultConfig().geeklistId + "'");					
				} 
				catch (NumberFormatException ex) {
					eventBroker.send(EventConstants.TOPIC_STATUS, "[ERROR] GeeklistId must be a whole number.");
				}
            }
        });
        cbGeeklistId.addFocusListener(new FocusListener() {
			
			@Override
			public void focusLost(FocusEvent e) {
				try {
					setGeeklistId(Integer.parseInt(cbGeeklistId.getText()));
	            	System.out.println("set geeklist id to '" + configManager.getResultConfig().geeklistId + "'");					
				} 
				catch (NumberFormatException ex) {
					eventBroker.send(EventConstants.TOPIC_STATUS, "[ERROR] GeeklistId must be a whole number.");
				}
					
			}
			
			@Override
			public void focusGained(FocusEvent e) {
				// nothing
			}
		});
        
        
        Label lblInfo = new Label(mainCompo, SWT.LEFT);
		lblInfo.setText("Description: ");
		lblInfo.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false));
		
        confLabel = new StyledText(mainCompo, SWT.MULTI | SWT.READ_ONLY | SWT.BORDER);
        confLabel.setText("");
        confLabel.setWordWrap(true);
        confLabel.setBounds(10,10,100,100);
        confLabel.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));       

        mainCompo.pack();
        mainCompo.layout(true, true);
	}

	public void setGeeklistId(Integer geeklistId) {
		if(eventBroker != null) {
			eventBroker.post(EventConstants.TOPIC_GEEKLIST_CHANGED, geeklistId);
		} else {
		   	System.out.println("setGeeklistId: eventBroker is null.");
		}
	}
	
	@Inject
	@Optional
	private void subscribeTopicGeeklistInfo
	    (@UIEventTopic(EventConstants.TOPIC_GEEKLIST_INFO)
	        String geeklistInfo) {
		System.out.println("TOPIC_GEEKLIST_INFO: " + geeklistInfo);
		confLabel.setText(geeklistInfo);
		confLabel.redraw();
	}
}
