package xyz.veronie.bgg.ui.filters.composites;

import org.eclipse.e4.core.services.events.IEventBroker;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.wb.swt.ResourceManager;

import xyz.veronie.bgg.result.ResultConfigManager;
import xyz.veronie.bgg.types.EventConstants;
import xyz.veronie.bgg.types.FilterFlagState;
import xyz.veronie.bgg.types.UserFlag;
import xyz.veronie.bgg.types.UserFlagEvent;

public class FilterFlagComposite extends Composite {

	private SwitchLabel flagSwitch;

	/**
	 * Create the composite.
	 * @param parent
	 * @param style
	 */
	public FilterFlagComposite(Composite parent, UserFlag flag,
			ResultConfigManager configManager, IEventBroker eventBroker) 
	{
		super(parent, SWT.NONE);

		this.setLayout(new GridLayout(2, false));
		this.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));

		// get current state from config to initialize
		FilterFlagState flagState = configManager.getResultConfig().userFlags.get(flag);
		boolean isTurnedOn = !flagState.equals(FilterFlagState.IGNORE);
		
		Button btnCheckFlag = new Button(this, SWT.CHECK);
		btnCheckFlag.setText(flag.toString());
		btnCheckFlag.setSelection(isTurnedOn);
		GridData gd_btnCheckFlag = new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1);
		gd_btnCheckFlag.widthHint = 120;
		btnCheckFlag.setLayoutData(gd_btnCheckFlag);

		
		flagSwitch = new SwitchLabel(this, SWT.NONE);
		flagSwitch.setOnImage(ResourceManager.getPluginImage("xyz.veronie.bgg.ui", "icons/noun_switch on_30x30.png"));
		flagSwitch.setOffImage(ResourceManager.getPluginImage("xyz.veronie.bgg.ui", "icons/noun_switch off_30x30.png"));


		flagSwitch.setEnabled(isTurnedOn);
		if(isTurnedOn) {
			if(flagState.equals(FilterFlagState.IS)) {
				setSwitchOn();				
			} else { // IS_NOT
				setSwitchOff();
			}
		} else {
			setSwitchOn();
		}

		flagSwitch.addMouseListener(new MouseListener() {
			@Override
			public void mouseDoubleClick(MouseEvent e) {
			}

			@Override
			public void mouseDown(MouseEvent e) {
			}

			@Override
			public void mouseUp(MouseEvent e) {
				UserFlagEvent ufe = new UserFlagEvent();
		    	ufe.flag = flag;

				boolean isSelected = !flagSwitch.getState();
	    		if(isSelected) {
	    			ufe.state = FilterFlagState.IS;
	    			setSwitchOn();
	    		} else {
	    			ufe.state = FilterFlagState.ISNOT;
	    			setSwitchOff();
	    		}

				eventBroker.send(EventConstants.TOPIC_USERFLAG_CHANGED, ufe);
			}
		});


		
		btnCheckFlag.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				Button source = (Button)e.getSource();
				boolean isSelected = source.getSelection();

				UserFlagEvent ufe = new UserFlagEvent();
		    	ufe.flag = flag;

		    	if(!isSelected) {
		    		ufe.state =  FilterFlagState.IGNORE;
		    		setSwitchOn();
		    		flagSwitch.setEnabled(false);
		    	} else {
		    		if(flagSwitch.getState()) {
		    			ufe.state = FilterFlagState.IS;
		    		} else {
		    			ufe.state = FilterFlagState.ISNOT;
		    		}
		    		flagSwitch.setEnabled(true);
		    	}
				eventBroker.send(EventConstants.TOPIC_USERFLAG_CHANGED, ufe);
			}
		});
		
		this.pack();
		this.getParent().layout(true, true);
	}

	private void setSwitchOn() {
		flagSwitch.setState(true);
	}
	private void setSwitchOff() {
		flagSwitch.setState(false);
	}


	@Override
	protected void checkSubclass() {
		// Disable the check that prevents subclassing of SWT components
	}

}
