 
package xyz.veronie.bgg.ui.handlers;

import java.sql.SQLException;
import java.util.List;

import javax.inject.Inject;

import org.eclipse.e4.core.commands.ECommandService;
import org.eclipse.e4.core.commands.EHandlerService;
import org.eclipse.e4.core.di.annotations.CanExecute;
import org.eclipse.e4.core.di.annotations.Execute;
import org.eclipse.e4.core.services.events.IEventBroker;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;

import xyz.veronie.bgg.result.ThingProvider;
import xyz.veronie.bgg.types.EventConstants;
import xyz.veronie.bgg.ui.dialogs.LoadGameListDialog;

@SuppressWarnings("restriction")
public class LoadGamelistHandler {
	
	@Inject
	ThingProvider thingProvider;
	
	@Execute
	public void execute(Shell shell, IEventBroker eventBroker, ECommandService commandService, EHandlerService handlerService) {
		
		try {
			List<String> thingLists = thingProvider.getThingListNames();
			
			if(thingLists.isEmpty()) {
				MessageBox msgBox = new MessageBox(shell, SWT.ICON_ERROR | SWT.OK);
				msgBox.setMessage("There are no saved game lists.");
				msgBox.open();
				return;
			}
			
			LoadGameListDialog loadDialog = new LoadGameListDialog(shell, thingLists);
			loadDialog.open();
			
			int returnCode = loadDialog.getReturnCode();
			if(returnCode == Dialog.OK) {
				String name = loadDialog.getSelectedName();
				try {
					thingProvider.loadList(name);
					eventBroker.post(EventConstants.TOPIC_RESULT_CHANGED, name);
				}
				catch (Exception e) {
					MessageBox msgBox = new MessageBox(shell, SWT.ICON_ERROR | SWT.OK);
					msgBox.setMessage("Could not load game list.");
					msgBox.open();
					e.printStackTrace();
				}
			}
			
		} catch (SQLException e) {
			MessageBox msgBox = new MessageBox(shell, SWT.ICON_ERROR | SWT.OK);
			msgBox.setMessage("Could not load game lists.");
			msgBox.open();
			e.printStackTrace();
		}
		
	}
	
	
	@CanExecute
	public boolean canExecute() {
		return thingProvider != null;
	}
		
}